@@ -1,6 +1,8 @@
 import xnat
 from mlops.utils.logger import logger
 from itertools import chain
+import mlflow
+import pandas as pd
 
 
 def xnat_build_dataset(xnat_configuration: dict, actions: list = None, flatten_output=True, test_batch: int = -1):
@@ -26,8 +28,9 @@ def xnat_build_dataset(xnat_configuration: dict, actions: list = None, flatten_o
             from random import sample
             project_subjects = sample(project.subjects, test_batch)
         else:
-            project_subjects = range(0, len(project.subjects))
+            project_subjects = project.subjects[:]
 
+        missing_data_log = []
         for subject_i in project_subjects:
             subject = project.subjects.data[subject_i.id]
             data_sample = {'subject_uri': project.subjects[subject.id].uri,
@@ -39,11 +42,15 @@ def xnat_build_dataset(xnat_configuration: dict, actions: list = None, flatten_o
                     data = []
                     for action, data_label in actions:
                         action_data = []
-                        #  No actions, just return a list of subject IDs and URIs
+
                         logger.debug(f"Running action: {action.__name__} on {subject.id}")
                         xnat_obj = action(project.subjects[subject.id])
 
                         if len(xnat_obj) == 0:
+                            missing_data_log.append({'subject_id': subject_i.id,
+                                                     'subject_label': subject_i.label,
+                                                     'failed_action': action})
+                            logger.warn(f'No data found for {subject_i}: action {action} removing sample')
                             raise Exception
 
                         for obj in xnat_obj:
@@ -53,8 +60,8 @@ def xnat_build_dataset(xnat_configuration: dict, actions: list = None, flatten_o
 
                         data.append(action_data)
 
-                except:
-                    logger.warn('No data found; removing sample.')
+                except Exception as e:
+                    logger.warn(f'No data found for {subject_i}; removing sample. Exception {e}')
                     continue
 
                 if flatten_output:
@@ -64,4 +71,8 @@ def xnat_build_dataset(xnat_configuration: dict, actions: list = None, flatten_o
 
             dataset.append((data_sample))
 
+        if missing_data_log:
+            pd.DataFrame(missing_data_log).to_csv('missing_data_log.csv')
+            mlflow.log_artifact('missing_data_log.csv')
+
     return dataset
